function pout=convexinterpol(k,p,margin);
% Finds the closest (l1 norm) set of arbitrage
% free prices approximating p
% Inequalities have to be fullfilled with margin

% Set problem params
m=length(k);
euc=eye(m);
A=[];b=[];

% Convexity constraints
for i=1:m
   for j=1:m
       if (i~=j)
           A=[A;[zeros(1,(i-1)*2),1,k(j),zeros(1,(m-i)*2),zeros(1,m)]-[zeros(1,(j-1)*2),1,k(j),zeros(1,(m-j)*2),zeros(1,m)]];
           b=[b;0];
       end
    end
end

% Monotonicity constraints (sign of the subgradients)
for i=1:m
    A=[A;[zeros(1,(i-1)*2),-1,zeros(1,1+(m-i)*2+m)]];
    b=[b;-margin];
    A=[A;[zeros(1,(i-1)*2+1),1,zeros(1,(m-i)*2+m)]];
    b=[b;-margin];
    A=[A;[zeros(1,(i-1)*2+1),-1,zeros(1,(m-i)*2+m)]];
    b=[b;1-margin];
end

% Price must be positive
for i=1:m
    A=[A;[zeros(1,(i-1)*2),-1,-k(i),zeros(1,(m-i)*2+m)]];
    b=[b;-margin];
end

% Pricing constraints
for i=1:m
    A=[A;[zeros(1,(i-1)*2),1,k(i),zeros(1,(m-i)*2),-euc(i,:)]];
    b=[b;p(i)];
    A=[A;[zeros(1,(i-1)*2),-1,-k(i),zeros(1,(m-i)*2),-euc(i,:)]];
    b=[b;-p(i)];
end

% Objective
c=[zeros(2*m,1);ones(m,1)];

% Solve using LINPROG
disp(['Problem size: ',num2str(size(A))]);
%[y,fval,EXITFLAG,OUTPUT,x]=LINPROG(c,A,b);       

% Solve using sedumi
K.l=size(A,1);
pars.fid=0;
[x,y]=sedumi(A',-c,b,K,pars);
fval=c'*y;

pout=[];
for i=1:m
    pout=[pout,y(2*(i-1)+1)+k(i)*y(2*(i-1)+2)];
end
